/* C interface example for ASL_zfc1fb , ASL_zfc1bf */
#include <stdio.h>
#include <stdlib.h>
#include "fftw3.h"
int main()
{
  int ld=16;
  int n;
  fftw_complex *c;
  int i;
  FILE *fp;
  fftw_plan p;

  fp = fopen( "1d.dat", "r" );
  if( fp == NULL )
  {
    printf( "file open error\n" );
    return -1;
  }

  printf( " *** ASL_zfc1fb , ASL_zfc1bf ***\n" );
  printf( "\n ** Input **\n\n" );
  c = fftw_malloc((size_t)( sizeof(fftw_complex) * ld ));

  if( c == NULL )
  {
    printf( "no enough memory for array c\n" );
    return -1;
  }
  fscanf( fp, "%d", &n );
  for( i=0 ; i<n ; i++ )
  {
    fscanf( fp, "%lf%lf", &c[i][0], &c[i][1] );
  }
  printf( "\t Real Part Imaginary Part\n" );
  for( i=0 ; i<n ; i++ )
  {
    printf( "\t c[%3d].r = %8.3g c[%3d].i = %8.3g\n", i, c[i][0], i, c[i][1] );
  }
  fclose( fp );

  printf( "\n ** Output **\n" );
  p = fftw_plan_dft_1d(ld, c, c, FFTW_FORWARD, FFTW_ESTIMATE);
  fftw_execute(p);
  
  for( i=0 ; i<n ; i++ )
  {
    c[i][0] /= n;
    c[i][1] /= n;
  }
  printf( "\n\t< Forward Transform >\n" );
  printf( "\n\tSolution\n\n" );
  printf( "\t Real Part Imaginary Part\n" );
  for( i=0 ; i<n ; i++ )
  {
    printf( "\t c[%3d].r = %8.3g c[%3d].i = %8.3g\n", i, c[i][0], i, c[i][1] );
  }

  p = fftw_plan_dft_1d(ld, c , c, FFTW_BACKWARD, FFTW_ESTIMATE);
  fftw_execute(p);

  printf( "\n\t< Backward Transform >\n" );
  printf( "\n\tSolution\n\n" );
  printf( "\t Real Part Imaginary Part\n" );
  for( i=0 ; i<n ; i++ )
  {
    printf( "\t c[%3d].r = %8.3g c[%3d].i = %8.3g\n", i, c[i][0], i, c[i][1] );
  }

  fftw_destroy_plan(p);
  fftw_free(c);
  fftw_cleanup();
  return 0;
}
