/* C Interface example for ASL_zfc2fb , ASL_zfc2bf */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fftw3.h>

int main()
{
  int nx = 5; int ny = 5;
  int lx = 5; int ly = 5;
  fftwf_complex *c;
  int i,j;
  fftw_plan p;

  printf( " *** ASL_zfc2fb , ASL_zfc2bf ***\n" );
  printf( "\n ** Input **\n" );
  c = ( fftwf_complex * )malloc((size_t)( sizeof(fftwf_complex) * (lx*ly) ));

  if( c == NULL )
  {
    printf( "no enough memory for array c\n" );
    return -1;
  }

  printf( "\tnx = %6d\n", nx );
  printf( "\tny = %6d\n", ny );

  for( j=1 ; j<=ny ; j++ )
  {
    for( i=1 ; i<=nx ; i++ )
    {
      c[(i-1)+lx*(j-1)][0]=(double)(i+j);
      c[(i-1)+lx*(j-1)][1]=(double)(i*j)/(double)(nx*ny);
    }
  }
  printf( "\tc[ix][iy]\n" );

  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j][0], c[i+lx*j][1] );
    }
    printf( "\n" );
  }

  p = fftw_plan_dft_2d(nx, ny, c, c, FFTW_FORWARD, FFTW_ESTIMATE);
  fftw_execute(p);

  for( i=0 ; i<lx*ly ; i++)
  {
    c[i][0] /= (double)( nx * ny );
    c[i][1] /= (double)( nx * ny );
  }

  printf( "\n ** Output **\n" );
  printf( "\t< Forward Transform >\n" );
  printf( "\tc[ix][iy]\n" );

  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j][0], c[i+lx*j][1] );
    }
    printf( "\n" );
  }

  p = fftw_plan_dft_2d(nx, ny, c, c, FFTW_BACKWARD, FFTW_ESTIMATE);
  fftw_execute(p);

  printf( "\t< Backward Transform >\n" );
  printf( "\tc[ix][iy]\n" );

  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j][0], c[i+lx*j][1] );
    }
    printf( "\n" );
  }

  fftw_free( c );
  fftw_destroy_plan(p);
  fftw_cleanup();

  return 0;
}
