/* C Interface example for ASL_zfc2fb , ASL_zfc2bf */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fftw3.h"

int main()
{
  int nx = 5; int ny = 4; int nz = 3;
  int lx = 5; int ly = 4; int lz = 3;
  fftw_complex *c;
  int i,j,k;
  fftw_plan p;

  printf( " *** ASL_zfc3fb , ASL_zfc3bf ***\n" );
  printf( "\n ** Input **\n" );
  c = ( fftw_complex * )fftw_malloc((size_t)( sizeof(fftw_complex) * (lx*ly*lz) ));

  if( c == NULL )
  {
    printf( "no enough memory for array c\n" );
    return -1;
  }

  printf( "\tnx = %6d\n", nx );
  printf( "\tny = %6d\n", ny );
  printf( "\tny = %6d\n", nz );

  for( k=1 ; k<=nz ; k++ )
  {
    for( j=1 ; j<=ny ; j++ )
    {
      for( i=1 ; i<=nx ; i++ )
      {
        c[(i-1)+lx*(j-1)+lx*ly*(k-1)][0]=(double)(nx+ny+nz)/(double)(i+j+k) ;
        c[(i-1)+lx*(j-1)+lx*ly*(k-1)][1]=(double)(i*j*k)/(double)(nx*ny*nz) ;
      }
    }
  }

  printf( "\tc[ix][iy][1]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j ][0], c[i+lx*j ][1] );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][2]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*1][0], c[i+lx*j+lx*ly*1][1] );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][3]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*2][0], c[i+lx*j+lx*ly*2][1] );
    }
  printf( "\n" );
  }

  p = fftw_plan_dft_3d(nx, ny, nz, c, c, FFTW_FORWARD, FFTW_ESTIMATE);
  fftw_execute(p);

  for( i=0 ; i<lx*ly*lz ; i++)
  {
    c[i][0] /= (double)( nx * ny * nz );
    c[i][1] /= (double)( nx * ny * nz );
  }

  printf( "\n ** Output **\n" );
  printf( "\t< Forward Transform >\n" );

  printf( "\tc[ix][iy][1]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j ][0], c[i+lx*j ][1] );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][2]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*1][0], c[i+lx*j+lx*ly*1][1] );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][3]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*2][0], c[i+lx*j+lx*ly*2][1] );
    }
    printf( "\n" );
  }

  p = fftw_plan_dft_3d(nx, ny, nz, c, c, FFTW_BACKWARD, FFTW_ESTIMATE);
  fftw_execute(p);

  printf( "\t< Backward Transform >\n" );

  printf( "\tc[ix][iy][1]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j ][0], c[i+lx*j ][1] );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][2]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*1][0], c[i+lx*j+lx*ly*1][1] );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][3]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*2][0], c[i+lx*j+lx*ly*2][1] );
    }
    printf( "\n" );
  }

  fftw_free( c );
  fftw_destroy_plan(p);
  fftw_cleanup();

  return 0;
}
