/* C Interface example for ASL_zfc2fb , ASL_zfc2bf */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mkl_dfti.h"

int main()
{
  int nx = 5; int ny = 5;
  int lx = 5; int ly = 5;
  MKL_Complex8 *c;
  int i,j;
  DFTI_DESCRIPTOR *my_desc1_handle;
  long status;
  long dm[2]={ny, nx};   /* MKL user may write {nx, ny, nz}. */

  printf( " *** ASL_cfc2fb , ASL_cfc2bf ***\n" );
  printf( "\n ** Input **\n" );
  c = ( MKL_Complex8 * )malloc((size_t)( sizeof(MKL_Complex8) * (lx*ly) ));

  if( c == NULL )
  {
    printf( "no enough memory for array c\n" );
    return -1;
  }

  printf( "\tnx = %6d\n", nx );
  printf( "\tny = %6d\n", ny );

  for( j=1 ; j<=ny ; j++ )
  {
    for( i=1 ; i<=nx ; i++ )
    {
      c[(i-1)+lx*(j-1)].real=(float)(i+j);
      c[(i-1)+lx*(j-1)].imag=(float)(i*j)/(float)(nx*ny);
    }
  }
  printf( "\tc[ix][iy]\n" );

  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j].real, c[i+lx*j].imag );
    }
    printf( "\n" );
  }

  status=DftiCreateDescriptor( &my_desc1_handle, DFTI_SINGLE, DFTI_COMPLEX, 2, dm);
  status=DftiCommitDescriptor( my_desc1_handle );
  status=DftiComputeForward( my_desc1_handle, c);
  status=DftiFreeDescriptor( &my_desc1_handle );

  for( i=0 ; i<lx*ly ; i++)
  {
    c[i].real /= (float)( nx * ny );
    c[i].imag /= (float)( nx * ny );
  }

  printf( "\n ** Output **\n" );
  printf( "\t< Forward Transform >\n" );
  printf( "\tc[ix][iy]\n" );

  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j].real, c[i+lx*j].imag );
    }
    printf( "\n" );
  }

  status=DftiCreateDescriptor( &my_desc1_handle, DFTI_SINGLE, DFTI_COMPLEX, 2, dm);
  status=DftiCommitDescriptor( my_desc1_handle );
  status=DftiComputeBackward( my_desc1_handle, c);
  status=DftiFreeDescriptor( &my_desc1_handle );

  printf( "\t< Backward Transform >\n" );
  printf( "\tc[ix][iy]\n" );

  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j].real, c[i+lx*j].imag );
    }
    printf( "\n" );
  }

  return 0;
}
